/*

This function performs the Error Analysis after Page Read operation 
is completed. This is done by calculating the Syndromes using 
the ECC generated during page write and page read.

Any one bit error detected, will be corrected in this routine.
If there is any error in the ECC itself, then the block is discarded
by updating the BadBlock Table.

*/


#include "Extern_Declaration.h"

extern int Stored_ECC[];
extern int Calc_ECC[];

int k = 0;
short ECC_Error_Flag =0;

/**********************************************************************/


void Error_Analysis()

{
	
	int syndrome0, syndrome1, syndrome2, syndrome3, syndrome4;
	short Failing_bit, Failing_Byte, Actual_Byte;

	/*
	The ECC for Write is in WECC buffer and that for read is in RECC.
	perform the error syndrome
	

	*/
	
	syndrome0 = (((Calc_ECC[k]<< 11) |  Calc_ECC[k+1]) ^ ((Stored_ECC[k]<< 11) |  Stored_ECC[k+1]));
	
	
	if(syndrome0 != 0)
	{
		if(countones(syndrome0) == 1)
		{
			ECC_Error_Flag = 1;
			printf("Error in ECC. Bad Block Table Updated \n");
		}
	
		else 
		{
			// Calculate the other syndromes.
		
			syndrome1 = Calc_ECC[k] ^ Stored_ECC[k];
	
			syndrome2 = Calc_ECC[k] ^ Calc_ECC[k+1];
	
			syndrome3 = Stored_ECC[k] ^ Stored_ECC[k+1] ;
	
			syndrome4 = syndrome2 ^ syndrome3;
			
			if((countones(syndrome0) == 11) && (syndrome4 == 0x7FF))
			{
			
				Failing_bit = syndrome1 & 7; 	// 0-2 indicate failing bit
				Failing_Byte = syndrome1 >> 3;	// 3-10 indicate failing byte
				Actual_Byte = *(ReadData + Failing_Byte);
				Actual_Byte = Actual_Byte ^ (0x1 << Failing_bit);
				*(ReadData + Failing_Byte) = Actual_Byte;
				printf("1 bit error detected and corrected \n");
			}
		
			else
			{
				printf("multiple error\n");
			}
		}
	}
	
		
	k++; // Update the pointer, so that next ECC value is read from buffer
	
}


/**********************************************************************/
